# -*- mode:Python; tab-width: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
###############################################################################
#              ______   ___            _______                                #
#             /\  _  \ /\_ \          /\______\                               #
#             \ \ \_\ \\//\ \        / / ___  /__    __                       #
#              \ \  __ \ \ \ \      / / /_\/ //\_\  /\_\                      #
#               \ \ \/\ \ \_\ \_   / / _____// / /_/ / /                      #
#                \ \_\ \_\/\____\ / / /      \/ /__\/ /                       #
#                 \/_/\/_/\/____/ \/_/       _\____  /                        #
#                                           /\___\/ /                         #
#                                           \/_____/                          #
#                                                                             #
#     AlPy: Python bindings to the Allegro game programming library           #
#                                                                             #
#     v 0.1.5, 2007-04-29.                                                    #
#                                                                             #
###############################################################################

# set to zero if your version of Allegro doesn't use dynamic loadable modules
_ALLEGRO_USE_MODULES = 1

import os, sys, imp

if os.name != 'posix':
    _ALLEGRO_USE_MODULES = 0

# to resolve external symbol references on Allegro modules on Unix.
# Thanks to Grzegorz Adam Hankiewicz for the suggestions.
# note: this is EXPERIMENTAL code (but it seems to work)
def _import_alpy():
    if not _ALLEGRO_USE_MODULES:
        return __import__('_alpy')
    _major, _minor = sys.version_info[:2]
    try: import DLFCN
    except ImportError:
        msg = "Sorry, to be able to use Allegro loadable modules you need to install the 'DLFCN' module: you can still use Alpy, but some features will not be available"
        print >> sys.stderr, "Warning: %s\n" % msg
        return __import__('_alpy')
    if _major > 2 or (_major == 2 and _minor >= 2):
        sys.setdlopenflags(DLFCN.RTLD_NOW|DLFCN.RTLD_GLOBAL)
        return __import__('_alpy')
    else:
        file, path, desc = imp.find_module('_alpy')
        try:
            handle = DLFCN.open(path, DLFCN.RTLD_NOW|DLFCN.RTLD_GLOBAL)
            m = __import__('_alpy')
            handle.close()
            return m
        finally:
            if file: file.close()
#end of _import_alpy

_alpy = _import_alpy()
del _import_alpy
from _alpy import *

AlpyException = error

def AL_ID(s, pos=None, n=0):
    x = 0
    for i in range(len(s)):
        if pos is not None and pos==i: val = ord(s[i]) + n
        else: val = ord(s[i])
        x = (x<<8) + val
    return x

OSTYPE_UNKNOWN   =  0
OSTYPE_WIN3      =  AL_ID('WIN3')
OSTYPE_WIN95     =  AL_ID('W95 ')
OSTYPE_WIN98     =  AL_ID('W98 ')
OSTYPE_WINME     =  AL_ID('WME ')
OSTYPE_WINNT     =  AL_ID('WNT ')
OSTYPE_WIN2000   =  AL_ID('W2K ')
OSTYPE_WINXP     =  AL_ID('WXP ')
OSTYPE_OS2       =  AL_ID('OS2 ')
OSTYPE_WARP      =  AL_ID('WARP')
OSTYPE_DOSEMU    =  AL_ID('DEMU')
OSTYPE_OPENDOS   =  AL_ID('ODOS')
OSTYPE_LINUX     =  AL_ID('TUX ')
OSTYPE_FREEBSD   =  AL_ID('FBSD')
OSTYPE_UNIX      =  AL_ID('UNIX')
OSTYPE_BEOS      =  AL_ID('BEOS')
OSTYPE_QNX       =  AL_ID('QNX ')
OSTYPE_MACOS     =  AL_ID('MAC ')

SYSTEM_AUTODETECT = 0
SYSTEM_NONE     = AL_ID('NONE')
SYSTEM_XWINDOWS = AL_ID('XWIN')
SYSTEM_LINUX    = AL_ID('LNXC')
SYSTEM_DIRECTX  = AL_ID('DX  ')
SYSTEM_BEOS     = AL_ID('BSYS')
SYSTEM_MACOS    = AL_ID('MCOS')
SYSTEM_QNX      = AL_ID('QSYS')

GFX_TEXT        = -1
GFX_AUTODETECT  = 0
GFX_AUTODETECT_FULLSCREEN = 1
GFX_AUTODETECT_WINDOWED   = 2
GFX_SAFE        = AL_ID('SAFE')
GFX_XWINDOWS    = AL_ID('XWIN')
GFX_XWINDOWS_FULLSCREEN = AL_ID('XWFS')
GFX_XDGA        = AL_ID('XDGA')
GFX_XDFS        = AL_ID('XDFS')
GFX_XDGA_FULLSCREEN = AL_ID('XDFS')
GFX_XDGA2       = AL_ID('DGA2')
GFX_XDGA2_SOFT  = AL_ID('DGAS')
GFX_VGA         = AL_ID('VGA ')
GFX_MODEX       = AL_ID('MODX')
GFX_FBCON       = AL_ID('FB  ')
GFX_VBEAF       = AL_ID('VBAF')
GFX_GGI         = AL_ID('GGI ')
GFX_SVGALIB     = AL_ID('SVGA')
GFX_GDI         = AL_ID('GDIB')
GFX_DIRECTX     = AL_ID('DXAC')
GFX_DIRECTX_ACCEL = GFX_DIRECTX
GFX_DIRECTX_SOFT = AL_ID('DXSO')
GFX_DIRECTX_SAFE = AL_ID('DXSA')
GFX_DIRECTX_WIN  = AL_ID('DXWN')
GFX_DIRECTX_OVL  = AL_ID('DXOV')
GFX_BEOS         = AL_ID('BFS ')
GFX_BEOS_FULLSCREEN = AL_ID('BFS ')
GFX_BEOS_FULLSCREEN_SAFE = AL_ID('BFSS')
GFX_BEOS_WINDOWED = AL_ID('BWN ')
GFX_BEOS_WINDOWED_SAFE = AL_ID('BWNS')
GFX_DRAWSPROCKET  = AL_ID('DSP ')
GFX_PHOTON       =  AL_ID('QPH ')
GFX_PHOTON_DIRECT = AL_ID('QPHD')

KEYBOARD_XWINDOWS       = AL_ID('XWIN')
KEYBOARD_LINUX          = AL_ID('LNXC')
KEYBOARD_DIRECTX        = AL_ID('DX  ')
KEYBOARD_BEOS           = AL_ID('BKEY')
KEYBOARD_MACOS          = AL_ID('MCOS')
KEYBOARD_QNX            = AL_ID('QKEY')

MOUSE_XWINDOWS  = AL_ID('XWIN')
MOUSE_DIRECTX   = AL_ID('DX  ')
MOUSE_BEOS      = AL_ID('BMOU')
MOUSE_MACOS     = AL_ID('MCOS')
MOUSE_QNX       = AL_ID('QMSE')

MOUSEDRV_LINUX_PS2      = AL_ID('LPS2')
MOUSEDRV_LINUX_GPMDATA  = AL_ID('GPMD')
MOUSEDRV_LINUX_MS       = AL_ID('MS  ')

JOY_TYPE_LINUX_ANALOGUE = AL_ID('LNXA')
JOY_TYPE_WIN32          = AL_ID('W32 ')
JOYSTICK_BEOS           = AL_ID('BJOY')


U_ASCII         = AL_ID('ASC8')
U_ASCII_CP      = AL_ID('ASCP')
U_UNICODE       = AL_ID('UNIC')
U_UTF8          = AL_ID('UTF8')
U_CURRENT       = AL_ID('cur.')

TIMERS_PER_SECOND = 1193181

def SECS_TO_TIMER(x):
    return x*TIMERS_PER_SECOND

def MSEC_TO_TIMER(x):
    return x*(TIMERS_PER_SECOND/1000)

def BPS_TO_TIMER(x):
    return TIMERS_PER_SECOND/x

def BPM_TO_TIMER(x):
    return (60*TIMERS_PER_SECOND)/x

# Keyboard constants
KEY_A              =  1
KEY_B              =  2
KEY_C              =  3
KEY_D              =  4
KEY_E              =  5
KEY_F              =  6
KEY_G              =  7
KEY_H              =  8
KEY_I              =  9
KEY_J              =  10
KEY_K              =  11
KEY_L              =  12
KEY_M              =  13
KEY_N              =  14
KEY_O              =  15
KEY_P              =  16
KEY_Q              =  17
KEY_R              =  18
KEY_S              =  19
KEY_T              =  20
KEY_U              =  21
KEY_V              =  22
KEY_W              =  23
KEY_X              =  24
KEY_Y              =  25
KEY_Z              =  26
KEY_0              =  27
KEY_1              =  28
KEY_2              =  29
KEY_3              =  30
KEY_4              =  31
KEY_5              =  32
KEY_6              =  33
KEY_7              =  34
KEY_8              =  35
KEY_9              =  36
KEY_0_PAD          =  37
KEY_1_PAD          =  38
KEY_2_PAD          =  39
KEY_3_PAD          =  40
KEY_4_PAD          =  41
KEY_5_PAD          =  42
KEY_6_PAD          =  43
KEY_7_PAD          =  44
KEY_8_PAD          =  45
KEY_9_PAD          =  46
KEY_F1             =  47
KEY_F2             =  48
KEY_F3             =  49
KEY_F4             =  50
KEY_F5             =  51
KEY_F6             =  52
KEY_F7             =  53
KEY_F8             =  54
KEY_F9             =  55
KEY_F10            =  56
KEY_F11            =  57
KEY_F12            =  58
KEY_ESC            =  59
KEY_TILDE          =  60
KEY_MINUS          =  61
KEY_EQUALS         =  62
KEY_BACKSPACE      =  63
KEY_TAB            =  64
KEY_OPENBRACE      =  65
KEY_CLOSEBRACE     =  66
KEY_ENTER          =  67
KEY_COLON          =  68
KEY_QUOTE          =  69
KEY_BACKSLASH      =  70
KEY_BACKSLASH2     =  71
KEY_COMMA          =  72
KEY_STOP           =  73
KEY_SLASH          =  74
KEY_SPACE          =  75
KEY_INSERT         =  76
KEY_DEL            =  77
KEY_HOME           =  78
KEY_END            =  79
KEY_PGUP           =  80
KEY_PGDN           =  81
KEY_LEFT           =  82
KEY_RIGHT          =  83
KEY_UP             =  84
KEY_DOWN           =  85
KEY_SLASH_PAD      =  86
KEY_ASTERISK       =  87
KEY_MINUS_PAD      =  88
KEY_PLUS_PAD       =  89
KEY_DEL_PAD        =  90
KEY_ENTER_PAD      =  91
KEY_PRTSCR         =  92
KEY_PAUSE          =  93
KEY_ABNT_C1        =  94
KEY_YEN            =  95
KEY_KANA           =  96
KEY_CONVERT        =  97
KEY_NOCONVERT      =  98
KEY_AT             =  99
KEY_CIRCUMFLEX     =  100
KEY_COLON2         =  101
KEY_KANJI          =  102

if get_allegro_c_version() >= (4,2,0):
    KEY_EQUALS_PAD   = 103
    KEY_BACKQUOTE    = 104
    KEY_SEMICOLON    = 105
    KEY_COMMAND      = 106
    KEY_UNKNOWN1     = 107
    KEY_UNKNOWN2     = 108
    KEY_UNKNOWN3     = 109
    KEY_UNKNOWN4     = 110
    KEY_UNKNOWN5     = 111
    KEY_UNKNOWN6     = 112
    KEY_UNKNOWN7     = 113
    KEY_UNKNOWN8     = 114
    KEY_MODIFIERS    = 115
    KEY_LSHIFT       = 115
    KEY_RSHIFT       = 116
    KEY_LCONTROL     = 117
    KEY_RCONTROL     = 118
    KEY_ALT          = 119
    KEY_ALTGR        = 120
    KEY_LWIN         = 121
    KEY_RWIN         = 122
    KEY_MENU         = 123
    KEY_SCRLOCK      = 124
    KEY_NUMLOCK      = 125
    KEY_CAPSLOCK     = 126
    KEY_MAX          = 127
else:
    KEY_MODIFIERS    = 103
    KEY_LSHIFT       = 103
    KEY_RSHIFT       = 104
    KEY_LCONTROL     = 105
    KEY_RCONTROL     = 106
    KEY_ALT          = 107
    KEY_ALTGR        = 108
    KEY_LWIN         = 109
    KEY_RWIN         = 110
    KEY_MENU         = 111
    KEY_SCRLOCK      = 112
    KEY_NUMLOCK      = 113
    KEY_CAPSLOCK     = 114
    KEY_MAX          = 115

KB_SHIFT_FLAG      = 0x0001
KB_CTRL_FLAG       = 0x0002
KB_ALT_FLAG        = 0x0004
KB_LWIN_FLAG       = 0x0008
KB_RWIN_FLAG       = 0x0010
KB_MENU_FLAG       = 0x0020
KB_SCROLOCK_FLAG   = 0x0100
KB_NUMLOCK_FLAG    = 0x0200
KB_CAPSLOCK_FLAG   = 0x0400
KB_INALTSEQ_FLAG   = 0x0800
KB_ACCENT1_FLAG    = 0x1000
KB_ACCENT2_FLAG    = 0x2000
KB_ACCENT3_FLAG    = 0x4000
KB_ACCENT4_FLAG    = 0x8000

TIMERDRV_UNIX   = AL_ID('UNIX')
TIMER_WIN32_ST  = AL_ID('W32S')
TIMER_WIN32_MT  = AL_ID('W32M')
TIMER_BEOS      = AL_ID('BTIM')
TIMER_MACOS     = AL_ID('MCOS')

DIGI_OSS        = AL_ID('OSSD')
DIGI_ESD        = AL_ID('ESDD')
DIGI_ALSA       = AL_ID('ALSA')
DIGI_DIRECTX    = lambda n: AL_ID('DXA ', 2, n)
DIGI_DIRECTAMX  = lambda n: AL_ID('AXA ', 2, n)
DIGI_WAVOUTID   = lambda n: AL_ID('WOA ', 2, n)
DIGI_BEOS       = AL_ID('BDIG')
DIGI_MACOS      = AL_ID('MCOS')

MIDI_OSS        = AL_ID('OSSM')
MIDI_WIN32MAPPER = AL_ID('W32M')
MIDI_WIN32      = lambda n: AL_ID('W32A', 3, n)
MIDI_BEOS       = AL_ID('BMID')
MIDI_QUICKTIME  = AL_ID('QT  ')

SWITCH_NONE             = 0
SWITCH_PAUSE            = 1
SWITCH_AMNESIA          = 2
SWITCH_BACKGROUND       = 3
SWITCH_BACKAMNESIA      = 4

SWITCH_IN       = 0
SWITCH_OUT      = 1

#drawing mode constants
DRAW_MODE_SOLID           = 0
DRAW_MODE_XOR             = 1
DRAW_MODE_COPY_PATTERN    = 2
DRAW_MODE_SOLID_PATTERN   = 3
DRAW_MODE_MASKED_PATTERN  = 4
DRAW_MODE_TRANS           = 5

# color conversion constants
COLORCONV_NONE            = 0

COLORCONV_8_TO_15         = 1
COLORCONV_8_TO_16         = 2
COLORCONV_8_TO_24         = 4
COLORCONV_8_TO_32         = 8

COLORCONV_15_TO_8         = 0x10
COLORCONV_15_TO_16        = 0x20
COLORCONV_15_TO_24        = 0x40
COLORCONV_15_TO_32        = 0x80

COLORCONV_16_TO_8         = 0x100
COLORCONV_16_TO_15        = 0x200
COLORCONV_16_TO_24        = 0x400
COLORCONV_16_TO_32        = 0x800

COLORCONV_24_TO_8         = 0x1000
COLORCONV_24_TO_15        = 0x2000
COLORCONV_24_TO_16        = 0x4000
COLORCONV_24_TO_32        = 0x8000

COLORCONV_32_TO_8         = 0x10000
COLORCONV_32_TO_15        = 0x20000
COLORCONV_32_TO_16        = 0x40000
COLORCONV_32_TO_24        = 0x80000

COLORCONV_32A_TO_8        = 0x100000
COLORCONV_32A_TO_15       = 0x200000
COLORCONV_32A_TO_16       = 0x400000
COLORCONV_32A_TO_24       = 0x800000

COLORCONV_DITHER_PAL      = 0x1000000
COLORCONV_DITHER_HI       = 0x2000000

COLORCONV_DITHER          = (COLORCONV_DITHER_PAL | COLORCONV_DITHER_HI)

COLORCONV_EXPAND_256      = (COLORCONV_8_TO_15 | COLORCONV_8_TO_16 |
                             COLORCONV_8_TO_24 | COLORCONV_8_TO_32)

COLORCONV_REDUCE_TO_256   = (COLORCONV_15_TO_8 | COLORCONV_16_TO_8 |
                             COLORCONV_24_TO_8 | COLORCONV_32_TO_8 |
                             COLORCONV_32A_TO_8)

COLORCONV_EXPAND_15_TO_16 =  COLORCONV_15_TO_16
COLORCONV_REDUCE_16_TO_15 =  COLORCONV_16_TO_15

COLORCONV_EXPAND_HI_TO_TRUE = (COLORCONV_15_TO_24 | COLORCONV_15_TO_32 |
                               COLORCONV_16_TO_24 | COLORCONV_16_TO_32)

COLORCONV_REDUCE_TRUE_TO_HI = (COLORCONV_24_TO_15 | COLORCONV_24_TO_16 |
                               COLORCONV_32_TO_15 | COLORCONV_32_TO_16)

COLORCONV_24_EQUALS_32      = (COLORCONV_24_TO_32 | COLORCONV_32_TO_24)

COLORCONV_TOTAL = (COLORCONV_EXPAND_256 | COLORCONV_REDUCE_TO_256 |
                   COLORCONV_EXPAND_15_TO_16 | COLORCONV_REDUCE_16_TO_15 |
                   COLORCONV_EXPAND_HI_TO_TRUE | COLORCONV_REDUCE_TRUE_TO_HI |
                   COLORCONV_24_EQUALS_32 | COLORCONV_32A_TO_15 |
                   COLORCONV_32A_TO_16 | COLORCONV_32A_TO_24)

COLORCONV_PARTIAL  = (COLORCONV_EXPAND_15_TO_16 | COLORCONV_REDUCE_16_TO_15 |
                      COLORCONV_24_EQUALS_32)

COLORCONV_MOST     = (COLORCONV_EXPAND_15_TO_16 | COLORCONV_REDUCE_16_TO_15 |
                      COLORCONV_EXPAND_HI_TO_TRUE |
                      COLORCONV_REDUCE_TRUE_TO_HI | COLORCONV_24_EQUALS_32)

allegro_id = ''
os_type = 0

# remove functions we don't want to expose
del install_allegro
del get_allegro_id
del get_os_type
del get_palette_color
#del allegro_init
del get_default_palette
del get_black_palette
del get_desktop_palette
del get_font
del set_gfx_mode
del get_screen
del get_key
del install_keyboard
del get_var, set_var


class CountersDict:
    """ A class to easily access user defined counters created
    by the register_counter function """
    def __getattr__(self, key):
        return _alpy.get_var(key)
    def __getitem__(self, key):
        try: return _alpy.get_var(key)
        except AttributeError, e: raise KeyError, str(e)
    def __setattr__(self, key, value):
        _alpy.set_var(key, value)
    def __setitem__(self, key, value):
        _alpy.set_var(key, value)

counters = CountersDict()
del CountersDict

def install_allegro(system_id):
    "install_allegro(system_id) -> None\n\nInitialises the Allegro library. You must call either this or\nallegro_init() before doing anything. Functions that can be called\nbefore this one will be marked explicitly in the documentation.\nRaises AlpyException on error.\n\nThe available system ID codes will vary from one platform\nto another, but you will almost always want to pass\nSYSTEM_AUTODETECT. Alternatively, SYSTEM_NONE installs a stripped\ndown version of Allegro that won't even try to touch your hardware\nor do anything platform specific: this can be useful for situations\nwhere you only want to manipulate memory bitmaps, such as the text\nmode datafile tools or the Windows GDI interfacing functions.\n\nExample:\n\n   import alpy\n   alpy.install_allegro(alpy.SYSTEM_AUTODETECT)"
    _alpy.install_allegro(system_id)

    global allegro_id, os_type
    allegro_id = _alpy.get_allegro_id()
    os_type = _alpy.get_os_type()

    global default_palette
    default_palette = _alpy.get_default_palette()
    global black_palette
    black_palette = _alpy.get_black_palette()
    global desktop_palette
    desktop_palette = _alpy.get_desktop_palette()
    global font
    font = _alpy.get_font()

    global palette_color
    class PaletteColorArray:
        def __getitem__(self, index):
            return _alpy.get_palette_color(index)
    palette_color = PaletteColorArray()

    # What are we doing here? This trick is to avoid an annoying situation:
    # if the user used "from alpy import *", the globals just created are not
    # visible, because at the import time they didn't exist; with the following
    # code we test if such kind of import has been used, and if so we export
    # the new variables. N.B.: The test is not so precise, beacuse for example
    # an "import alpy as al" statement will cause the names to be expored even
    # if they shouldn't, but actually I have no idea how to make it better.
    m = sys.modules['__main__']
    if not hasattr(m, __name__):
        setattr(m, 'default_palette', default_palette)
        setattr(m, 'desktop_palette', desktop_palette)
        setattr(m, 'black_palette', black_palette)
        setattr(m, 'font', font)
        setattr(m, 'palette_color', palette_color)

def allegro_init():
    "allegro_init() -> None\n\nInitialises the Allegro library. This is the same thing as\ncalling install_allegro(SYSTEM_AUTODETECT). Raises AlpyException\non failure."
    install_allegro(SYSTEM_AUTODETECT)

def set_gfx_mode(card, w, h, v_w = 0, v_h = 0):
    "set_gfx_mode(card, w, h, v_w = 0, v_h = 0) -> None\n\nSwitches into graphics mode. The card parameter should usually be\nGFX_AUTODETECT, GFX_AUTODETECT_FULLSCREEN or GFX_AUTODETECT_WINDOWED,\nor see the platform specific documentation for a list of the\navailable drivers. The w and h parameters specify what screen\nresolution you want. The color depth of the graphic mode has to be\nspecified before calling this function with set_color_depth().\n\nThe v_w and v_h parameters specify the minimum virtual screen size,\nin case you need a large virtual screen for hardware scrolling or\npage flipping. You should set them to zero if you don't care about\nthe virtual screen size.\n\nWhen you call set_gfx_mode(), the v_w and v_h parameters represent\nthe minimum size of virtual screen that is acceptable for your\nprogram. The range of possible sizes is usually very restricted,\nand Allegro is likely to end up creating a virtual screen much larger\nthan the one you request.  On an SVGA card with one megabyte of vram\nyou can count on getting a 1024x1024 virtual screen (256 colors)\nor 1024x512 (15 or 16 bpp), and with 512k vram you can get 1024x512\n(256 colors). Other sizes may or may not be possible: don't assume\nthat they will work. In mode-X the virtual width can be any multiple\nof eight greater than or equal to the physical screen width, and the\nvirtual height will be set accordingly (the VGA has 256k of vram,\nso the virtual height will be 256*1024/virtual_width).\n\nAfter you select a graphics mode, the global variables SCREEN_W,\nSCREEN_H, VIRTUAL_W, and VIRTUAL_H will tell you the real and\nvirtual screen size.\n\nThis function will raise alpy.AlpyException if the requested graphic\nmode cannot be set. Example:\n\n   try:\n      alpy.set_gfx_mode(alpy.GFX_AUTODETECT, 320, 200)\n   except alpy.AlpyException:\n      # Couldn't set graphic mode, abort!\n      sys.exit(1)"
    _alpy.set_gfx_mode(card, w, h, v_w, v_h)
    if card != GFX_TEXT:
        global screen, SCREEN_W, SCREEN_H, VIRTUAL_W, VIRTUAL_H
        screen = _alpy.get_screen()
        SCREEN_W, SCREEN_H, VIRTUAL_W, VIRTUAL_H = _alpy.get_screen_size()
        m = sys.modules['__main__']
        if not hasattr(m, __name__):
            setattr(m, 'screen', screen)
            setattr(m, 'SCREEN_W', SCREEN_W)
            setattr(m, 'SCREEN_H', SCREEN_H)
            setattr(m, 'VIRTUAL_W', VIRTUAL_W)
            setattr(m, 'VIRTUAL_H', VIRTUAL_H)

def install_keyboard():
    "install_keyboard() -> int\n\nInstalls the Allegro keyboard interrupt handler. You must call\nthis before using any of the keyboard input routines. Once you have\nset up the Allegro handler, you can no longer use operating system\ncalls or C library functions to access the keyboard. This function\nis called by default by alpy.init().\n\nNote that on some platforms the keyboard won't work unless you\nhave set a graphics mode, even if this function returns a success\nvalue before calling set_gfx_mode. This can happen in environments\nwith graphic windowed modes, since Allegro usually reads the\nkeyboard through the graphical window (which appears after the\nset_gfx_call). Example:\n\n   alpy.install_allegro(alpy.SYSTEM_AUTODETECT)\n   alpy.install_timer()\n   alpy.install_keyboard()\n   # We are not 100% we can read the keyboard yet!\n   alpy.set_gfx_mode(alpy.GFX_AUTODETECT, 640, 480)\n\n   # Now we are guaranteed to be able to read the keyboard.\n   alpy.readkey()\n\nReturns zero on success, or a negative number on failure (but you\nmay decide not to check the return value as this function is very\nunlikely to fail)."
    r = _alpy.install_keyboard()
    if r == 0:
        global key
        class KeyArray:
            def __getitem__(self,index):
                return _alpy.get_key(index)
        key = KeyArray()
    m = sys.modules['__main__']
    if not hasattr(m, __name__):
        setattr(m, 'key', key)
    return r

def xor_mode(xor):
    "xor_mode(bool) -> None\n\nThis is a shortcut for toggling xor drawing mode on and off.\nCalling xor_mode(True) is equivalent to drawing_mode(DRAW_MODE_XOR,\nNone, 0, 0).  Calling xor_mode(False) is equivalent to\ndrawing_mode(DRAW_MODE_SOLID, None, 0, 0)."
    if xor != 0:
        drawing_mode(DRAW_MODE_XOR, None, 0, 0)
    else:
        drawing_mode(DRAW_MODE_SOLID, None, 0, 0)

def solid_mode():
    "solid_mode() -> None\n\nThis is a shortcut for selecting solid drawing mode. It is equivalent\nto calling drawing_mode(DRAW_MODE_SOLID, None, 0, 0)."
    drawing_mode(DRAW_MODE_SOLID, None, 0, 0)

#blender functions constants
TRANS      =  0
ADD        =  1
BURN       =  2
COLOR      =  3
DIFFERENCE =  4
DISSOLVE   =  5
DODGE      =  6
HUE        =  7
INVERT     =  8
LUMINANCE  =  9
MULTIPLY   = 10
SATURATION = 11
SCREEN     = 12

def set_trans_blender(*args):
    "set_trans_blender(int r, int g, int b, int a) -> None\n\nEnables a linear interpolator blender mode for combining translucent\nor lit truecolor pixels.\n\nThis is the same as calling set_blender(TRANS, ...)."
    set_blender(TRANS, *args)

def set_add_blender(*args):
    "set_add_blender(int r, int g, int b, int a) -> None\n\nEnables an additive blender mode for combining translucent or lit\ntruecolor pixels.\n\nThis is the same as calling set_blender(ADD, ...)."
    set_blender(ADD, *args)

def set_burn_blender(*args):
    "set_burn_blender(int r, int g, int b, int a) -> None\n\nEnables a burn blender mode for combining translucent or lit\ntruecolor pixels. Here the lightness values of the colours of\nthe source image reduce the lightness of the destination image,\ndarkening the image.\n\nThis is the same as calling set_blender(BURN, ...)."
    set_blender(BURN, *args)

def set_color_blender(*args):
    "set_color_blender(int r, int g, int b, int a) -> None\n\nEnables a color blender mode for combining translucent or lit\ntruecolor pixels. Applies only the hue and saturation of the source\nimage to the destination image. The luminance of the destination\nimage is not affected.\n\nThis is the same as calling set_blender(COLOR, ...)."
    set_blender(COLOR, *args)

def set_difference_blender(*args):
    "set_difference_blender(int r, int g, int b, int a) -> None\n\nEnables a difference blender mode for combining translucent or lit\ntruecolor pixels. This makes an image which has colours calculated\nby the difference between the source and destination colours.\n\nThis is the same as calling set_blender(DIFFERENCE, ...)."
    set_blender(DIFFERENCE, *args)

def set_dissolve_blender(*args):
    "set_dissolve_blender(int r, int g, int b, int a) -> None\n\nEnables a dissolve blender mode for combining translucent or lit\ntruecolor pixels. Randomly replaces the colours of some pixels in\nthe destination image with those of the source image. The number\nof pixels replaced depends on the alpha value (higher value, more\npixels replaced; you get the idea :).\n\nThis is the same as calling set_blender(DISSOLVE, ...)."
    set_blender(DISSOLVE, *args)

def set_dodge_blender(*args):
    "set_dodge_blender(int r, int g, int b, int a) -> None\n\nEnables a dodge blender mode for combining translucent or lit\ntruecolor pixels. The lightness of colours in the source lighten\nthe colours of the destination. White has the most effect; black\nhas none.\n\nThis is the same as calling set_blender(DODGE, ...)."
    set_blender(DODGE, *args)

def set_hue_blender(*args):
    "set_hue_blender(int r, int g, int b, int a) -> None\n\nEnables a hue blender mode for combining translucent or lit truecolor\npixels. This applies the hue of the source to the destination.\n\nThis is the same as calling set_blender(HUE, ...)."
    set_blender(HUE, *args)

def set_invert_blender(*args):
    "set_invert_blender(int r, int g, int b, int a) -> None\n\nEnables an invert blender mode for combining translucent or lit\ntruecolor pixels. Blends the inverse (or negative) colour of the\nsource with the destination.\n\nThis is the same as calling set_blender(INVERT, ...)."
    set_blender(INVERT, *args)

def set_luminance_blender(*args):
    "set_luminance_blender(int r, int g, int b, int a) -> None\n\nEnables a luminance blender mode for combining translucent or\nlit truecolor pixels. Applies the luminance of the source to the\ndestination.  The colour of the destination is not affected.\n\nThis is the same as calling set_blender(LUMINANCE, ...)."
    set_blender(LUMINANCE, *args)

def set_multiply_blender(*args):
    "set_multiply_blender(int r, int g, int b, int a) -> None\n\nEnables a multiply blender mode for combining translucent or lit\ntruecolor pixels. Combines the source and destination images,\nmultiplying the colours to produce a darker colour. If a colour is\nmultiplied by white it remains unchanged; when multiplied by black\nit also becomes black.\n\nThis is the same as calling set_blender(MULTIPLY, ...)."
    set_blender(MULTIPLY, *args)

def set_saturation_blender(*args):
    "set_saturation_blender(int r, int g, int b, int a) -> None\n\nEnables a saturation blender mode for combining translucent or\nlit truecolor pixels. Applies the saturation of the source to the\ndestination image.\n\nThis is the same as calling set_blender(SATURATION, ...)."
    set_blender(SATURATION, *args)

def set_screen_blender(*args):
    "set_screen_blender(int r, int g, int b, int a) -> None\n\nEnables a screen blender mode for combining translucent or lit\ntruecolor pixels. This blender mode lightens the colour of the\ndestination image by multiplying the inverse of the source and\ndestination colours. Sort of like the opposite of the multiply\nblender mode.\n\nThis is the same as calling set_blender(SCREEN, ...)."
    set_blender(SCREEN, *args)

def set_write_alpha_blender():
    "set_write_alpha_blender() -> None\n\nShortcut which calls set_alpha_blender(1)."
    set_alpha_blender(1)

def create_light_table(pal, r, g, b, callback = None):
    "create_light_table(pal, r, g, b, callback = None) -> table\n\nCreates and returns a color mapping table with lookup data for\ndoing lighting effects with the specified palette. When combining\nthe colors c1 and c2 with this table, c1 is treated as a light\nlevel from 0-255. At light level 255 the table will output color\nc2 unchanged, at light level 0 it will output the r, g, b value\nyou specify to this function, and at intermediate light levels it\nwill output a color somewhere between the two extremes. The r, g,\nand b values are in the range 0-63.\n\nIf the callback function is not None, it will be called 256\ntimes during the calculation, allowing you to display a progress\nindicator. This callback function will be passed a parameter\nindicating the position of the process as an integer from 0 to 255.\n\nThis is the same as calling:\n\n   create_color_map(pal, (r, g, b), callback, type=\"light\")."
    return create_color_map(pal, (r, g, b), callback, type="light")

def create_trans_table(pal, r, g, b, callback = None):
    "create_trans_table(pal, r, g, b, callback = None) -> table\n\nCreates and returns a color mapping table with lookup data for doing\ntranslucency effects with the specified palette. When combining\nthe colors c1 and c2 with this table, the result will be a color\nsomewhere between the two. The r, g, and b parameters specify\nthe solidity of each color component, ranging from 0 (totally\ntransparent) to 255 (totally solid). For 50% solidity, pass 128.\nThis function treats source color #0 as a special case, leaving the\ndestination unchanged whenever a zero source pixel is encountered,\nso that masked sprites will draw correctly.\n\nIf the callback function is not None, it will be called 256\ntimes during the calculation, allowing you to display a progress\nindicator. This callback function will be passed a parameter\nindicating the position of the process as an integer from 0 to 255.\n\nThis is the same as calling:\n\n   create_color_map(pal, (r, g, b), callback, type=\"trans\")."
    return create_color_map(pal, (r, g, b), callback, type="trans")

def create_color_table(pal, blend, callback = None):
    "create_color_table(pal, blend, callback = None) -> table\n\nCreates and returns a color mapping table with lookup data for doing\na paletted equivalent of whatever truecolor blender mode is currently\nselected. After calling set_trans_blender(), set_blender_mode(),\nor any of the other truecolor blender mode routines, you can use\nthis function to create an 8-bit mapping table that will have the\nsame results as whatever 24-bit blending mode you have enabled.\n\nIf the callback function is not None, it will be called 256\ntimes during the calculation, allowing you to display a progress\nindicator. This callback function will be passed a parameter\nindicating the position of the process as an integer from 0 to 255.\n\nThis is the same as calling:\n\n   create_color_map(pal, (r, g, b), callback, type=\"blend\")."
    return create_color_map(pal, blend, callback, type="blend")

def init(keyboard=1, mouse=1, timer=1):
    "init(keyboard = 1, mouse = 1, timer = 1) -> None\n\nHandy shortcut which calls allegro_init() and then depending on\nthe boolean parameters install_keyboard(), install_mouse() and\ninstall_timer() respectively (by default all are called)."
    allegro_init()
    if keyboard: install_keyboard()
    if mouse: install_mouse()
    if timer: install_timer()

def Datafile(filename, obj_name=None):
    "Datafile(filename, obj_name = None) -> Datafile or [Datafile, ...]\n\nConstructor for datafiles. When the obj_name parameter is none, the\nwhole datafile will be loaded, returning a list of Datafile objects.\nIf you pass a valid obj_name, only a single Datafile object will\nbe returned.\n\nThe function will raise AlpyException if there are problems."
    if obj_name is None: return load_datafile(filename)
    return load_datafile_object(filename, obj_name)

